import numpy as np

cfg = {
    'name': 'NIRS - Cerebral Oximeter Index',
    'group': 'Medical algorithms',
    'desc': 'Calculate Pearson correlation coefficient between blood pressure and cerebral oxymetry',
    'reference': 'Brady. et al. Continuous time-domain analysis of cerebrovascular autoregulation using near-infrared spectroscopy. Stroke. 2007 October; 38(10):2818-2825.',
    'interval': 300,
    'overlap': 270,
    'inputs': [
        {'name': 'ART_MBP', 'type': 'num'},
        {'name': 'SCO2_R', 'type': 'num'}
        ],
    'outputs': [
        {'name': 'cox_slope', 'type':'num', 'unit':'%/mmHg', 'min': -1, 'max': 1},
        {'name': 'cox_pearson', 'type':'num', 'min':-1, 'max':1},
        {'name': 'cox_ncc', 'type':'num', 'min':-1, 'max':1},
        {'name': 'mean_mbp', 'type':'num', 'unit':'mmHg'}
        ]
}


def run(inp, opt, cfg):
    dt_last = cfg['interval']

    med_interval = 10

    sco_avgvals = []  # 10 sec mean
    mbp_avgvals = []
    for dt_bin_from in range(0, dt_last, med_interval):
        dt_bin_to = min(dt_bin_from + med_interval, dt_last)
        scos = []  # collect for 10 sec
        mbps = []
        for sco in inp['SCO2_R']:
            if 20 < sco['val']:
                if dt_bin_from < sco['dt'] < dt_bin_to:
                    scos.append(sco['val'])
        for mbp in inp['ART_MBP']:
            if 20 < mbp['val'] < 200:
                if dt_bin_from < mbp['dt'] < dt_bin_to:
                    mbps.append(mbp['val'])
        if not mbps or not scos:
            continue
        sco_avgvals.append(np.mean(scos))
        mbp_avgvals.append(np.mean(mbps))

    if len(mbp_avgvals) == 0:
        return
#
#    if np.max(mbp_avgvals) - np.min(mbp_avgvals) < 20:
#        return

    mbp = np.median(mbp_avgvals)
    pr = np.corrcoef(mbp_avgvals, sco_avgvals)[0, 1]
    slope, _ = np.linalg.lstsq(np.vstack([mbp_avgvals, np.ones(len(mbp_avgvals))]).T, sco_avgvals)[0]

    # normalized cross correlation
    a = mbp_avgvals
    b = sco_avgvals
    a = (a - np.mean(a)) / (np.std(a) * len(a))
    b = (b - np.mean(b)) / (np.std(b))
    xr = np.max(np.correlate(a, b, 'full'))

    return [
        [{'dt': dt_last, 'val': slope}],
        [{'dt': dt_last, 'val': pr}],
        [{'dt': dt_last, 'val': xr}],
        [{'dt': dt_last, 'val': mbp}]
    ]
